structure Tokens = Tokens

type pos = int
type svalue = Tokens.svalue
type ('a, 'b) token = ('a, 'b) Tokens.token
type lexresult = (svalue, pos) token

exception Error
val pos = ref 1
fun inc x = x := ((!x) + 1)
val error = fn x => (print("ERROR (Line "^(Int.toString (!pos))^"): " ^ x ^ "\n"); raise Error)
val eof = fn () => Tokens.EOF(!pos, !pos)

%%

%header (functor LambdaLexFun(structure Tokens : Lambda_TOKENS));

alpha = [A-Za-z];
digit = [0-9];
ws = [\ \t];


%%
\n	=> (inc pos; lex());
{ws}+ 	 => (lex());
"+"	 => (Tokens.PLUS(!pos, !pos));
"*"	 => (Tokens.STAR(!pos, !pos));
"("	 => (Tokens.LPAREN(!pos, !pos));
")"	 => (Tokens.RPAREN(!pos, !pos));
","	 => (Tokens.COMMA(!pos, !pos));
"."      => (Tokens.PERIOD(!pos, !pos));
":"	 => (Tokens.COLON(!pos, !pos));
"="	 => (Tokens.EQUALS(!pos, !pos));
"true" 	 => (Tokens.TRUE(!pos, !pos));
"false"  => (Tokens.FALSE(!pos, !pos));
"if"	 => (Tokens.IF(!pos, !pos));
"then"	 => (Tokens.THEN(!pos, !pos));
"else"	 => (Tokens.ELSE(!pos, !pos));
"fst"	 => (Tokens.FST(!pos, !pos));
"snd"	 => (Tokens.SND(!pos, !pos));
"lambda" => (Tokens.LAMBDA(!pos, !pos));
"let"	 => (Tokens.LET(!pos, !pos));
"in"	 => (Tokens.IN(!pos, !pos));
"end"	 => (Tokens.END(!pos, !pos));
"int"	 => (Tokens.INTTYP(!pos, !pos));
"bool"	 => (Tokens.BOOLTYP(!pos, !pos));
"->"	 => (Tokens.ARROW(!pos, !pos));
{alpha}+ => (Tokens.VAR(yytext, !pos, !pos));
~?{digit}+ => (case (Int.fromString yytext) of NONE => (error "Bad integer")
				              | SOME n => (Tokens.INT(n, !pos, !pos)));
.	=> (error ("Invalid character found during lexing: " ^ yytext));
